#
#	This package uses the methods of Clauset, Shalizi and Newman (2009)
#	to test whether we have a power-law with greater refinement.
#
#	https://stats.stackexchange.com/questions/108843/how-to-test-whether-a-distribution-follows-a-power-law
#-------------------------------------------------------------------
library(poweRlaw)
library(methods)

args 	<- commandArgs(trailingOnly = TRUE)
#
#	Defines the maximum range for the exponential binning.
#
nbreaks	<- as.numeric(args[1])

indexOfLastNonZero <- function(x) {
	ix	= 0
	for(i in 1:length(x)) {
		ix = i
        	if (x[i] == 0) {
			ix = i
               break
        	}
     } 

	return(ix-1)
}
#
#	Read in the frequency data from the same length range as the adjusted R^2 was
#	computed, 300 <->30000
#
data <- read.table("length_powerlawtest.txt",header=FALSE)
x    <- data$V1
y    <- data$V2
#
#	The poweRlaw package just takes a vector but it must be as a rank order so evenly
#	sampled in x.
#
h <- hist(y,breaks=nbreaks,plot=FALSE)
xb   <- h$mid
yb   <- h$density

ybt	<- yb[1:indexOfLastNonZero(yb)]

plobj	<- conpl$new(ybt)
#
#	Estimate the xmin and the exponent α of the power law, and assign them to the power law object
#
est <- estimate_xmin(plobj,xmax=1e+05)
plobj$xmin <- est$xmin
plobj$pars <- est$pars
print("Power-law parameters, xmin, pars(alpha)")
plobj$xmin
plobj$pars
#
#	Also, at this point, you can see the KS statistic.
#
print("KS statistic, (max distance between data and power-law distribution")
est$KS
#
#	KS statistic tells you how well power law distribution fits your data, but it 
#	doesn't tell you how likely your data is drawn from power law. So you also need a p value. 
#	This is how you do it:
#
print("Estimated p-value.  Should be bigger than 0.1 for plausibility.")
bs <- bootstrap_p(plobj,xmax=1e+05,no_of_sims=1000)
bs$p
